"""Middleware to report application status to the infrastructure."""
import logging
import os

from django.http import HttpResponse
from django.http import JsonResponse


class HealthCheckMiddleware:
    """Intercepts ``GET`` requests on ``/.well-known/probe/live``
    and ``/.well-known/probe/ready`` to report application health.
    """
    logger = logging.getLogger("probe")

    def __init__(self, get_response):
        self.get_response = get_response

    def __call__(self, request):
        response = None
        if request.method == "GET":
            if request.path == "/.well-known/health/ready":
                response = self.ready(request)
            elif request.path == "/.well-known/health/live":
                response = self.live(request)
            elif request.path == '/.well-known/metadata':
                response = self.metadata(request)
            elif request.path == '/.well-known/commit':
                response = self.commit(request)
        return response or self.get_response(request)

    def commit(self, request):
        """Return the commit from which the current runtime was built."""
        return HttpResponse(os.getenv('VCS_COMMIT_HASH') or 'noop',
            content_type="text/plain")

    def live(self, request):
        """Respond with ``200 OK``."""
        self.logger.info("Received liveness probe from %s",
            request.META['REMOTE_ADDR'])
        return HttpResponse("OK")

    def metadata(self, request):
        """Return a datastructure with information about the current
        application.
        """
        dto = {
            'commit': os.getenv('VCS_COMMIT_HASH')
        }
        return JsonResponse(dto)

    def ready(self, request):
        """Respond with ``200 OK`` if all services are operational."""
        self.logger.info("Received readyness probe from %s",
            request.META['REMOTE_ADDR'])
        return HttpResponse("OK")



"""Common HTTP and browser related settings."""
#pylint: disable=no-member
import os
import unimatrix.lib.environ


ALLOWED_HOSTS = unimatrix.lib.environ.parselist(os.environ,
    'HTTP_ALLOWED_HOSTS', sep=';')

CSRF_COOKIE_PATH = os.getenv('HTTP_CSRF_COOKIE_PATH') or '/'

CSRF_COOKIE_DOMAIN = os.getenv('HTTP_CSRF_COOKIE_DOMAIN')

CSRF_COOKIE_SECURE = os.getenv('HTTP_CSRF_COOKIE_INSECURE') != '1'

CSRF_COOKIE_HTTPONLY = os.getenv('HTTP_CSRF_COOKIE_ALLOWHTTP') != '1'

FORCE_SCRIPT_NAME = os.getenv('HTTP_MOUNT_PATH')

SECURE_HSTS_SECONDS = 86400 * 365

SECURE_HSTS_PRELOAD = os.getenv('HTTP_HSTS_PRELOAD') == '1'

SESSION_COOKIE_DOMAIN = os.getenv('HTTP_SESSION_COOKIE_DOMAIN')

SESSION_COOKIE_NAME = os.getenv('HTTP_SESSION_COOKIE_NAME') or 'sessionid'

SESSION_COOKIE_PATH = os.getenv('HTTP_SESSION_COOKIE_PATH') or '/'

SESSION_COOKIE_SECURE = os.getenv('HTTP_SESSION_COOKIE_INSECURE') != '1'

SESSION_COOKIE_HTTPONLY = os.getenv('HTTP_SESSION_COOKIE_ALLOWHTTP') != '1'



"""Dynamically constructs a Django settings module based on the operating
platform environment with consistent defaults.
"""
#pylint: disable=unused-wildcard-import,invalid-name,wildcard-import
#pylint: disable=wrong-import-position,no-member
import importlib
import os
import sys
import inspect

import unimatrix.const
import unimatrix.lib.environ
from unimatrix.const import RUNDIR
from unimatrix.ext.django.lib import etc
from unimatrix.ext.django.settings.email import *
from unimatrix.ext.django.settings.rdbms import DATABASES
from unimatrix.ext.django.settings.http import *


__all__ = []


if 'UNIMATRIX_SETTINGS_MODULE' not in os.environ:
    raise RuntimeError('Unable to load UNIMATRIX_SETTINGS_MODULE')

WSGI_APPLICATION = 'unimatrix.ext.django.wsgi.application'

# Retrieve the current module from the sys.modules dictionary;
# we can then dynamically copy over settings from the real
# Django settings module.
self = sys.modules[__name__]
base_settings = importlib.import_module(os.environ['UNIMATRIX_SETTINGS_MODULE'])

# Iterate over all attributes in the original settings module
# and set them as attributes.
for attname, value in inspect.getmembers(base_settings):
    if attname == 'INSTALLED_APPS' and 'unimatrix.ext.django' not in value:
        value = tuple(['unimatrix.ext.django'] + list(value))
    if attname == 'MIDDLEWARE':
        if 'unimatrix.ext.django.lib.middleware.HealthCheckMiddleware'\
        not in value:
            value.insert(0,
                'unimatrix.ext.django.lib.middleware.HealthCheckMiddleware')
    setattr(self, attname, value)


# The following settings are hard-coded and needed for proper
# deployment on the Unimatrix platform.
from unimatrix.ext.django.settings.logging import *
from unimatrix.ext.django.settings.const import *


# Below members are operational configurations that are enforced
# by the Unimatrix platform. Since they are mandatory for deployment,
# we have the assignments raise an exception if the keys do
# not exist.
API_BROWSER_ENABLED = os.getenv('API_BROWSER_ENABLED') == '1'

API_BROWSER_PATH = os.getenv('API_BROWSER_PATH') or 'browse/'

CELERY_BROKER_URL = os.getenv('CELERY_BROKER_URL')

DEBUG = os.getenv('DEBUG') == '1'

DEPLOYMENT_ENV = os.getenv('DEPLOYMENT_ENV') or 'testing'

ROOT_URLCONF = 'unimatrix.ext.django.urls.runtime'

SECRET_KEY = os.getenv('SECRET_KEY') or ('0' * 64)

SESSION_ENGINE = 'django.contrib.sessions.backends.cache'
if os.getenv('SESSIONS_PERSISTENT') == '1':
    raise NotImplementedError

STATIC_ROOT = os.getenv('STATIC_ROOT') or os.path.join(RUNDIR, 'static')

STATIC_SERVE = os.getenv('STATIC_SERVE') == '1'

STATIC_URL = os.getenv('STATIC_URL') or '/assets/'
if not str.endswith(STATIC_URL, '/'):
    STATIC_URL = STATIC_URL + '/'


# We check here if DEBUG is True and the SECRET_KEY consist
# of all zeroes, to prevent insecure keys getting deployed
# in a production environment.
if (not DEBUG and SECRET_KEY == ('0' * 64))\
and (DEPLOYMENT_ENV != 'build'):
    raise RuntimeError("Insecure SECRET_KEY configured.")



"""Sets up the logging configuration for a Django project."""
import os


__all__ = ['LOGGING']

DEFAULT_LOG_LEVEL = 'ERROR'

SYSLOG_HOST = os.getenv('SYSLOG_HOST') or 'localhost'

SYSLOG_PORT = int(os.getenv('SYSLOG_PORT') or 5140)

SYSLOG_ENABLED = os.getenv('LOG_ENABLE_SYSLOG') == '1'

LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'formatters': {
        'verbose': {
            'format': "%(asctime)s [%(process)d] [%(levelname)s] %(message)s",
            'datefmt': "[%Y-%m-%d %H:%M:%S %z]"
        },
        'simple_syslog': {
            'format': "%(message)s\n"
        }
    },
    'handlers': {
        'console': {
            'class': 'logging.StreamHandler',
            'formatter': 'verbose'
        },
        'null': {
            'class': 'logging.NullHandler',
        },
        'syslog': {
            'level': 'DEBUG',
            'class': 'logging.handlers.SysLogHandler',
            'facility': 'user',
            'formatter': 'simple_syslog',
            'address' : (SYSLOG_HOST, SYSLOG_PORT),
        },
    },
    'loggers': {
        'django': {
            'handlers': ['console'],
            'level': os.getenv('DJANGO_LOG_LEVEL', DEFAULT_LOG_LEVEL),
        },
        'django.request': {
            'handlers': ['syslog', 'console'],
            'level': os.getenv('DJANGO_LOG_LEVEL', DEFAULT_LOG_LEVEL),
            'propagate': False,
        },
        'django.db.backends': {
            'handlers': ['null'],
            'propagate': False,
        },
        'proton': {
            'handlers': ['console'],
            'level': os.getenv('DJANGO_LOG_LEVEL', DEFAULT_LOG_LEVEL),
        },
        'probe': {
            'handlers': ['syslog', 'console'],
            'propagate': True,
            'level': os.getenv('DJANGO_LOG_LEVEL', DEFAULT_LOG_LEVEL),
        },
        'aorta': {
            'handlers': ['syslog', 'console'],
            'level': os.getenv('DJANGO_LOG_LEVEL', DEFAULT_LOG_LEVEL),
        },
        'drone': {
            'handlers': ['syslog', 'console'],
            'level': os.getenv('DJANGO_LOG_LEVEL', DEFAULT_LOG_LEVEL),
        },
    },
}


# Remove the syslog handler if it is not enabled.
if not SYSLOG_ENABLED:
    LOGGING['handlers']['syslog'] = LOGGING['handlers']['null']



"""Core Django application for Unimatrix projects."""
#pylint: disable=invalid-name
# This will make sure the app is always imported when
# Django starts so that shared_task will use this app.
from .celery import app as __celery__


__all__ = ['__celery__']

default_app_config = 'unimatrix.ext.django.apps.UnimatrixConfig'



"""Constants used in the Unimatrix One/CM platforms,
for deployments for Django.
"""
import os


SESSIONS_PERSISTENT = os.getenv('SESSIONS_PERSISTENT') == '1'

STATICFILES_DIRS = []
if os.path.exists('dist/assets'):
    STATICFILES_DIRS.append('dist/assets')

STATIC_ROOT = 'static'



"""Declares :class:`TemplateService`."""
import functools
import operator

import django.template.loader


class TemplateService:
    """Provides an interface to render templates using the Django
    template framework.
    """
    default_prefix = None

    def __init__(self, prefix=None):
        self.prefix = prefix or self.default_prefix

    def abspath(self, template_name):
        """Return a list holding the full paths to `template_name`."""
        template_names = []
        if isinstance(template_name, (list, tuple)):
            return functools.reduce(operator.add,
                [self.abspath(x) for x in template_name], [])

        if self.prefix:
            template_names.append(self._prefix_template_name(template_name))
        template_names.append(template_name)
        return template_names

    def get_template(self, template_name, using=None):
        """Load the given template and return a renderable :class:`Template`
        object.
        """
        return self._select_template(self.abspath(template_name), using=using)

    def render_to_string(self, template_name, ctx=None, request=None, using=None):
        """Renders the template `template_name` to a string."""
        template = self.get_template(template_name, using=using)
        return template.render(ctx, request)

    def _select_template(self, template_names, using=None):
        return django.template.loader.select_template(
            template_names, using=using)

    def _prefix_template_name(self, template_name):
        assert self.prefix
        return str.rstrip(self.prefix, '/') + '/'\
            + str.lstrip(template_name, '/')



"""Declares :class:`EmailService`."""
import ioc
from django.core.mail import send_mail
from django.conf import settings


class EmailService:
    """Provides an interface to send transactional emails."""

    def send(self, sender, subject, recipients, text, html):
        """Sends an email to the specified recipients."""
        if sender is None:
            sender = settings.DEFAULT_FROM_EMAIL
        return send_mail(subject, text, sender, recipients,
            html_message=html)

    @ioc.inject('template', 'TemplateService')
    def send_from_templates(self, sender, subject, recipients, text, html,
        ctx, template):
        """Sends an email to the specified recipients using templates."""
        if not text and not html:
            raise ValueError("Specify at least one of `text`, `html`.")
        if text:
            text = template.render_to_string(text, ctx)
        if html:
            html = template.render_to_string(html, ctx)
        return self.send(sender, subject, recipients, text, html)
